######################################################################################
######################################################################################

# Parent E. and Rivot E. - An introduction to Hierarchcial Modeling of Ecological Data
# Chapman et al.
# 2012

# BRUGS program for Hierarchical Stock-recruitment analysis 
# (with covariates Latitude)
# Chapter 9 - section 9.3

#######################################################################################
#######################################################################################


rm(list = ls())
library(boot)
library("BRugs")
library(coda)
library(MASS)



# --------------------------------------------------------------------
# MCMC config.
# --------------------------------------------------------------------


# BunrIn
NburnIn <- 10000
# Inferences
Niter <- 10000
thin <- 10
Nchains <- 3



# --------------------------------------------------------------------
# Model with 13 rivers treated as independent
# --------------------------------------------------------------------


data <- "data_HierarchicalSR.txt"
inits <- "inits_HierarchicalSR_Ind.txt"

model <- "model_HierarchicalSR_Ind.txt"
modelCheck(model)  
modelData(data)   
modelCompile(numChains=Nchains)      
modelInits(c(inits,inits,inits))

# Burn-in period

modelUpdate(NburnIn) 

# Inferences

monitor <- c("Sst","hst","tau")
samplesSet(monitor)
modelUpdate(numUpdates = Niter, thin = thin)

# Store MCMC samples of parameters in data.frames

Sst_ind <- as.data.frame(as.matrix(buildMCMC("Sst"))) 
hst_ind <- as.data.frame(as.matrix(buildMCMC("hst")))





# --------------------------------------------------------------------
# Hierarchical model with latitude as a covariate
# --------------------------------------------------------------------


data <- "data_HierarchicalSR.txt"
data_Lat <- "data_HierarchicalSR_Lat.txt"		# Additionnal data with latitude
inits <- "inits_HierarchicalSR_Hier.txt"
model <- "model_HierarchicalSR_Hier.txt"

modelCheck(model)  
modelData(data)
modelData(data_Lat)
modelCompile(numChains=Nchains)      
modelInits(c(inits,inits,inits))

# Burn-in period

modelUpdate(NburnIn) 

# Inferences
monitor <- c("Sst","hst","Sst.pred","hst.pred","sigma.h","alpha","beta","delta","kappa","sigma")
samplesSet(monitor)
modelUpdate(numUpdates = Niter, thin = thin)           

# Store MCMC samples of parameters in data.frames

Sst_hbm <- as.matrix(buildMCMC("Sst")) 
hst_hbm <- as.matrix(buildMCMC("hst"))
Sst_hbm.pred <- as.matrix(buildMCMC("Sst.pred")) 
hst_hbm.pred <- as.matrix(buildMCMC("hst.pred"))

alpha <- as.matrix(buildMCMC("alpha"))
beta <- as.matrix(buildMCMC("beta"))
delta <- as.matrix(buildMCMC("delta"))
kappa <- as.matrix(buildMCMC("kappa"))
sigma <- as.matrix(buildMCMC("sigma"))
sigma.h <- as.matrix(buildMCMC("sigma.h"))



# --------------------------------------------------------------------
# Results
# --------------------------------------------------------------------


# ------------------------------------------------
# Table estimates
# ------------------------------------------------

theta<-cbind(alpha,beta,delta,kappa,sigma,sigma.h,Sst_hbm,hst_hbm)

store.result <- function(x, ...)
{  
y<-(c(mean(x),var(x)^0.5,quantile(x,pr=c(0.025,0.975))))
#print(floor(10*y)/10)
res<-y
}                             
apply(theta,2,store.result) -> tabres
tabres <- data.frame(round(t(tabres),2))
names(tabres) <- c("mean","sd","2.5%","97.5%")
row.names(tabres) <- c("alpha","beta","delta","kappa","sigma","sigma.h")

tabres



# ------------------------------------------------
# Figures
# ------------------------------------------------


data <- source("data_HierarchicalSR.txt")$value
n <- data$n.riv
riv <- data$riv
S <- data$S
R <- data$R
n.obs <- data$n.obs
n.riv <- data$n.riv

data <- source("data_HierarchicalSR_Lat.txt")$value
lat <- data$lat
lat.pred <- data$lat.pred

size.labels = 1
size.text = 1.3
col1 <- "grey90"
col2 <- "grey55"
col1 = c(rep(col1, times=n))
col2 = c(rep(col2, times=n))
box.size1 = 0.4
box.size2 = 0.6

names_ind <- c("Nivelle","Oir","Frome","Dee","Burrishoole","Lune","Bush",
"Mourne","Faughan","Girnock Burn","North Esk","Laerdalselva","Ellidaar")

names_pred <- c("Nivelle","new 45","Oir","new 50","Frome","Dee","Burrishoole","Lune","new 55","Bush",
"Mourne","Faughan","Girnock Burn","North Esk","new 60","Laerdalselva","Ellidaar","new 65")




# ----------------------------------------------
# Fitted SR relationships
# ----------------------------------------------

# windows()

res <- 6
name_figure <- "FittedSR_13rivers.png"
png(filename = name_figure, height = 700*res, width = 500*res, res=72*res)

par(mfrow=c(5,3), bty = "n", mar = c(2,2,2,2))
def.par <- par(no.readonly = TRUE)

lwd1 <- 1; lty1 <- 1
lwd2 <- 3; lty2 <- 1
xlabel <- ""
ylabel <- ""
size.names <- 0.9
Splot <- seq(0,100,length.out=1000)
nn <- c(1,cumsum(n.obs))

for (i in 1:n.riv)
{

hopt.ind <- median(hst_ind[,i])
Sopt.ind <- median(Sst_ind[,i])
hopt.hbm <- median(hst_hbm[,i])
Sopt.hbm <- median(Sst_hbm[,i])

Rplot1 <- exp( log(Splot) + hopt.ind - log(1-hopt.ind) - (hopt.ind/Sopt.ind)*Splot )
Rplot2 <- exp( log(Splot) + hopt.hbm - log(1-hopt.hbm) - (hopt.hbm/Sopt.hbm)*Splot )

x.max <- signif(max(S[nn[i]:nn[i+1]])*1.5,digits=0)
y.max <- max(R[nn[i]:nn[i+1]])*1.5

plot(Splot, Rplot1 ,ylab = "", xlab = "", xaxt="n", yaxt="n", main="", xlim=c(0,x.max), ylim=c(0,y.max), col = "black", type = "l", lty = lty1, lwd = lwd1, xaxs="i")
points(Splot, Rplot2 ,ylab = "", xlab = "", xaxt="n", yaxt="n", main="", xlim = c(0,x.max), col = "black", type = "l", lty = lty2, lwd = lwd2)

axis(side = 1, tick = T, cex.axis=1, las=1, at=signif(seq(0,x.max,length.out=5),digits=2))
axis(side = 2, tick = T, cex.axis=1)
mtext(side=1, xlabel, bty="n",line=3, cex=1)
mtext(side=2, ylabel, bty="n", line=3, cex=1)
mtext(paste(names_ind[i]), line = 0, side = 3, cex = size.names)

points(S[nn[i]:nn[i+1]],R[nn[i]:nn[i+1]], pch=19, cex = 1)
}

dev.off()



# ----------------------------------------------
# log(S_star)
# ----------------------------------------------

# windows()

res <- 6
name_figure <- "Sst_ind_vs_hbm.png"
png(filename = name_figure, height = 500*res, width = 650*res, res=72*res)

par(mfrow = c(1,1), bty="n", mar=c(6,5,1,1))

X1 <- log(Sst_ind)
X2 <- log(Sst_hbm)

labels <- names_ind
x.min = 1 ; x.max = n+1
y.min = -2 ; y.max = 6
y.label = "log(S*)"
x.label = "Rivers"
title = ""

boxplot(	X1[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex=box.size1, col = col1[1]) 
for (i in 2:n) {
boxplot(X1[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex=box.size1, col = col1[i]) }

for (i in 1:n) {
boxplot(X2[,i], xaxt = "n", yaxt = "n", at = i+0.4, add = T, outpch = NA, boxwex=box.size1, col = col2[i]) }

axis(side =1, at=1:n, labels = labels, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
# mtext(x.label, line = 3, side = 1)
mtext(y.label, line = 3, side = 2, cex = size.text)
mtext(title, line = 1.5, side = 3, adj=0, cex = size.text)

dev.off()




# ----------------------------------------------
# h_star
# ----------------------------------------------

# windows()

res <- 6
name_figure <- "hst_ind_vs_hbm.png"
png(filename = name_figure, height = 500*res, width = 650*res, res=72*res)

par(mfrow = c(1,1), bty="n", mar=c(6,5,1,1))

X1 <- hst_ind
X2 <- hst_hbm

labels <- names_ind
x.min = 1 ; x.max = n+1
y.min = 0 ; y.max = 1
y.label = "h*"
x.label = "Rivers"
title = ""

boxplot(	X1[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex=box.size1, col = col1[1]) 
for (i in 2:n) {
boxplot(X1[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex=box.size1, col = col1[i]) }

for (i in 1:n) {
boxplot(X2[,i], xaxt = "n", yaxt = "n", at = i+0.4, add = T, outpch = NA, boxwex=box.size1, col = col2[i]) }

axis(side =1, at=1:n, labels = labels, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
# mtext(x.label, line = 3, side = 1)
mtext(y.label, line = 3, side = 2, cex = size.text)
mtext(title, line = 1.5, side = 3, adj=0, cex = size.text)

dev.off()




# ----------------------------------------------
# Marginal distribution of the 6 coeficients
# ----------------------------------------------


coef <- cbind.data.frame(alpha,beta,delta,kappa,sigma,sigma.h)
names <- c(expression(alpha),expression(beta),expression(delta),expression(kappa),expression(sigma),expression(tau))

# windows()

res <- 5
name_figure <- "Marginal_param_hbm.png"
png(filename = name_figure, height = 1000*res, width = 750*res, res=72*res)

par(mfrow=c(3,2), bty = "n", mar = c(3,3,3,3))
def.par <- par(no.readonly = TRUE)

col <- "black"
lwd1 <- 3; lty1 <- 1
xlabel <- ""
ylabel <- ""
size.text <- 1.5

for (i in 1:6)
{
X <- coef[,i]
dens <- density(X,ad=2)
x.min <- signif(quantile(X, probs = c(0.001)),digits=1)
x.max <- signif(quantile(X, probs = c(0.999)),digits=1)
y.max <- max(dens$y)*1.1

plot(dens, ,ylab = "", xlab = "", xaxt="n", yaxt="n", main="", xlim=c(x.min,x.max), ylim=c(0,y.max), col = col, type = "l", lty = lty1, lwd = lwd1, xaxs="i")

axis(side = 1, tick = T, cex.axis=1.2, las=1, at=signif(seq(x.min,x.max,length.out=5),digits=2))
axis(side = 2, tick = F, cex.axis=1.2, labels = F, lwd.ticks = 0)
mtext(side=1, xlabel, bty="n", line=3, cex=1)
mtext(side=2, ylabel, bty="n", line=3, cex=1)
mtext(names[i], line = 0, side = 3, cex = size.text)

}

dev.off()



# ----------------------------------------------
# Boxplot - log(S_star) - with latitude & prediction
# ----------------------------------------------

# windows()

res <- 6
name_figure <- "Sst_hbm_pred_lat.png"
png(filename = name_figure, height = 500*res, width = 650*res, res=72*res)

par(mfrow = c(1,1), bty="n", mar=c(6,5,1,1))

size.labels = 1
size.text = 1.3
box.size1 = 1
box.size2 = 1

X1 <- log(Sst_hbm)
X2 <- log(Sst_hbm.pred)

a <- median(alpha)
b <- median(beta)
lat.plot <- 40:70
log.Sst.plot <- a*lat.plot+b

n <- length(lat)
n.pred <- length(lat.pred)

labels <- names_pred

x.min = 40 ; x.max = 70
y.min = -2 ; y.max = 5
y.label = "log(S*)"
x.label = "Latitude"
title = ""

boxplot(	X1[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = lat[1], outpch = NA, boxwex=box.size1, col = col1[1]) 
for (i in 2:n) {
boxplot(X1[,i], xaxt = "n", yaxt = "n", at = lat[i], add = T, outpch = NA, boxwex=box.size1, col = col1[i]) }

for (i in 1:n.pred) {
boxplot(X2[,i], xaxt = "n", yaxt = "n", at = lat.pred[i], add = T, outpch = NA, boxwex=box.size2, col = col2[i]) }

points(lat.plot, log.Sst.plot, type="l",lty=1, lwd=1, col="black")

axis(side =1, at=signif(seq(x.min,x.max,length.out=11),digits=2), labels = T, las=1, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 3, side = 2, cex = size.text)
mtext(title, line = 1.5, side = 3, adj=0, cex = size.text)

dev.off()




# ----------------------------------------------
# Boxplot - h_star - with latitude & prediction
# ----------------------------------------------

# windows()

res <- 6
name_figure <- "hst_hbm_pred_lat.png"
png(filename = name_figure, height = 500*res, width = 650*res, res=72*res)

par(mfrow = c(1,1), bty="n", mar=c(6,5,1,1))

size.labels = 1
size.text = 1.3
box.size1 = 1
box.size2 = 1

X1 <- hst_hbm
X2 <- hst_hbm.pred

c <- median(delta)
d <- median(kappa)
lat.plot <- 40:70
logit.hst.plot <- c*lat.plot+d
hst.plot <- exp(logit.hst.plot)/(1+exp(logit.hst.plot))

n <- length(lat)
n.pred <- length(lat.pred)

labels <- names_pred

x.min = 40 ; x.max = 70
y.min = 0 ; y.max = 1
y.label = "h*"
x.label = "Latitude"
title = ""

boxplot(	X1[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = lat[1], outpch = NA, boxwex=box.size1, col = col1[1]) 
for (i in 2:n) {
boxplot(X1[,i], xaxt = "n", yaxt = "n", at = lat[i], add = T, outpch = NA, boxwex=box.size1, col = col1[i]) }

for (i in 1:n.pred) {
boxplot(X2[,i], xaxt = "n", yaxt = "n", at = lat.pred[i], add = T, outpch = NA, boxwex=box.size2, col = col2[i]) }

points(lat.plot, hst.plot, type="l",lty=1, lwd=1, col="black")

axis(side =1, at=signif(seq(x.min,x.max,length.out=11),digits=2), labels = T, las=1, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 3, side = 2, cex = size.text)
mtext(title, line = 1.5, side = 3, adj=0, cex = size.text)

dev.off()





# ----------------------------------------------
# Marginal - hopt
# ----------------------------------------------

# windows()

res <- 6
name_figure <- "Marginal_hopt.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

par(mfrow=c(5,3), bty = "n", mar = c(2,2,2,2))
def.par <- par(no.readonly = TRUE)

col1 <- "red"
col2 <- "blue"
lwd1 <- 1; lty1 <- 1
lwd2 <- 3; lty2 <- 1
xlabel <- ""
ylabel <- ""
size.text <- 0.9

nn <- c(1,cumsum(n.obs))

for (i in 1:n.riv)
{
X1 <- hst_ind[,i]
X2 <- hst_hbm[,i]
dens1 <- density(X1)
dens2 <- density(X2)
x.max <- 1
y.max <- max(max(dens1$y),max(dens2$y))*1.1

plot(dens1, ,ylab = "", xlab = "", xaxt="n", yaxt="n", main="", xlim=c(0,x.max), ylim=c(0,y.max), col = col1, type = "l", lty = lty1, lwd = lwd1)
points(dens2, ylab = "", xlab = "", xaxt="n", yaxt="n", main="", col = col2, type = "l", lty = lty2, lwd = lwd2)

axis(side = 1, tick = T, cex.axis=1, las=1)
axis(side = 2, tick = F, cex.axis=1, labels = F, lwd.ticks = 0)
mtext(side=1, xlabel, bty="n",line=3, cex=1)
mtext(side=2, ylabel, bty="n", line=3, cex=1)
mtext(paste(names_ind[i]), line = 0, side = 3, cex = size.text)

}

dev.off()


# ----------------------------------------------
# Marginal - Sopt
# ----------------------------------------------

# windows()

res <- 6
name_figure <- "Marginal_Sopt.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

par(mfrow=c(5,3), bty = "n", mar = c(2,2,2,2))
def.par <- par(no.readonly = TRUE)

col3 <- "red"
col4 <- "blue"
lwd1 <- 1; lty1 <- 1
lwd2 <- 3; lty2 <- 1
xlabel <- ""
ylabel <- ""
size.text <- 0.9
nn <- c(1,cumsum(n.obs))

for (i in 1:n.riv)
{
X1 <- Sst_ind[,i]
X2 <- Sst_hbm[,i]
dens1 <- density(X1)
dens2 <- density(X2)
x.max <- signif(quantile(X2, probs = c(0.995)),digits=1)
# x.max <- round(quantile(X2, probs = c(0.995)),digits=0)

y.max <- max(max(dens1$y),max(dens2$y))*1.1

plot(dens1, ,ylab = "", xlab = "", xaxt="n", yaxt="n", main="", xlim=c(0,x.max), ylim=c(0,y.max), col = col3, type = "l", lty = lty1, lwd = lwd1, xaxs="i")
points(dens2, ylab = "", xlab = "", xaxt="n", yaxt="n", main="", col = col4, type = "l", lty = lty2, lwd = lwd2)

axis(side = 1, tick = T, cex.axis=1, las=1, at=signif(seq(0,x.max,length.out=5),digits=2))
axis(side = 2, tick = F, cex.axis=1, labels = F, lwd.ticks = 0)
mtext(side=1, xlabel, bty="n", line=3, cex=1)
mtext(side=2, ylabel, bty="n", line=3, cex=1)
mtext(paste(names_ind[i]), line = 0, side = 3, cex = size.text)

}

dev.off()
